#!/bin/bash

#
# Atari Gamer Common Lynx Library.
# Copyright 2022 Igor Kromin (atarigamer.com)
#
# Licensed under the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License. 
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License.
#

# This script makes it easier to process BMP files that are the source
# of multi-sprite data i.e. when a BMP is split up into multiple sprites.
# Every sprite has a .spr file generated but only ONE .s file is generated
# for the entire sprite set. Best used with LITERAL sprites to make offset
# calculations easier e.g.
#   sprFontPtr->data = &img_ag_font5x5[11 * (_agFontStrChar)];

OBJ_NAME=$(basename "$1" | cut -f1 -d'.')
S_FILE="build/$OBJ_NAME.s"

echo "Processing sprite tiles for $OBJ_NAME"
touch "$S_FILE"

echo ".global _$OBJ_NAME" > "$S_FILE"
echo ".segment \"RODATA\"" >> "$S_FILE"
echo "_$OBJ_NAME:" >> "$S_FILE"

ls build/$OBJ_NAME*.spr | sort | while read file
do
    echo -n .
    
    spr=$(basename "$file")
    obj=$(echo "$spr" | cut -f1 -d'.')
#
#    echo ".global _$obj" >> "$S_FILE"
#    echo ".segment \"RODATA\"" >> "$S_FILE"
#    echo "_$obj: .incbin \"$spr\"" >> "$S_FILE"    

    echo " .incbin \"$spr\"" >> "$S_FILE"    
done
echo